#!/usr/bin/python3

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Program: Computer Info
# Written by Antti Luukkonen <antti.luukkonen@helsinki.fi>
# 23.3.2017


import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk as gtk
from gi.repository import Gdk
import subprocess
import os

VERSION = '0.3'

class InfoWin:

    def __init__(self):

        # Computer Name
        self.name = subprocess.check_output("hostname", shell=True).strip().decode('utf-8')
        self.hname = "Computer name: "

        # Username
        self.user = subprocess.check_output("whoami", shell=True).strip().decode('utf-8')
        self.huser = "Username: "

        # CPU
        self.cpu = subprocess.check_output("grep 'model name' /proc/cpuinfo | cut -f 2 | cut -c 3- | tail -1", shell=True).strip().decode('utf-8')
        self.hcpu = "CPU: "

        # Memory
        self.mem = subprocess.check_output("free -m | awk 'FNR == 2 {print $2}'", shell=True).strip().decode('utf-8')
        self.hmem = "RAM (MB): "

        # IP Address
        if len(subprocess.check_output("hostname -I", shell=True)) > 17:
            self.ips = subprocess.check_output("hostname -I", shell=True).decode('utf-8').split()
            self.ip = self.ips[0] + "\t" + self.ips[1]
        else:
            self.ip = subprocess.check_output("hostname -I", shell=True).strip().decode('utf-8')
        self.hip = "IP address: "

        # FQDN
        self.fqdn = subprocess.check_output("hostname --fqdn", shell=True).strip().decode('utf-8')
        self.hfqdn = "DNS address: "

        # MAC Address
        self.mac = subprocess.check_output("cat /sys/class/net/*/address | awk 'FNR == 1 {print}' | sed 's/://g'", shell=True).upper().strip().decode('utf-8')
        self.hmac = "MAC address: "
        
        # Serial number
        self.serial = subprocess.check_output("cat /var/lib/cubbli/hostinfo | grep DMI_chassis_serial | cut -d = -f 2", shell=True).upper().strip().decode('utf-8')
        self.hserial = "Serial number: "

        # Warranty date
        self.warranty = subprocess.check_output("if [ -f /etc/cubbli/computer-warranty ];then cat /etc/cubbli/computer-warranty; fi", shell=True).upper().strip().decode('utf-8')
        self.hwarranty = "Warranty expires: "

        # OS Version
        self.osv = subprocess.check_output("lsb_release -d | cut -f 2", shell=True).strip().decode('utf-8')
        self.hosv = "Operating system: "

        # Last Updated
        self.lupdate = subprocess.check_output("tail -1 /var/log/dpkg.log | awk '{print $1, $2}'", shell=True).strip().decode('utf-8')
        self.hlupdate = "Last updated: "

        # Local Drives
        self.ldrives = subprocess.check_output("df -h / /home", shell=True).strip().decode('utf-8')
        self.hldrives = "Local drives: "

        # Create a new window
        self.window = gtk.Window()
        self.window.set_title("Computer Info")

        # Close the window
        self.window.connect("destroy", lambda w: gtk.main_quit())

        # Size of the window
        self.window.set_size_request(550, 340)
        #self.window.set_position(gtk.WIN_POS_CENTER)

        self.vbox = gtk.Box(orientation=gtk.Orientation.VERTICAL)
        self.window.add(self.vbox)
        self.window.set_border_width(10)

        # Create a table for the labels
        table = gtk.Table(11, 2, True)

        # Attach labels to the table
        table.attach(self.create_label(self.hname), 0, 1, 0, 1, ypadding=5)
        table.attach(self.create_label(self.name), 1, 2, 0, 1, ypadding=5)
        table.attach(self.create_label(self.hcpu), 0, 1, 1, 2, ypadding=5)
        table.attach(self.create_label(self.cpu), 1, 2, 1, 2, ypadding=5)
        table.attach(self.create_label(self.hmem), 0, 1, 2, 3, ypadding=5)
        table.attach(self.create_label(self.mem), 1, 2, 2, 3, ypadding=5)
        table.attach(self.create_label(self.hosv), 0, 1, 3, 4, ypadding=5)
        table.attach(self.create_label(self.osv), 1, 2, 3, 4, ypadding=5)
        table.attach(self.create_label(self.hlupdate), 0, 1, 4, 5, ypadding=5)
        table.attach(self.create_label(self.lupdate), 1, 2, 4, 5, ypadding=5)
        table.attach(self.create_label(self.hfqdn), 0, 1, 5, 6, ypadding=5)
        table.attach(self.create_label(self.fqdn), 1, 2, 5, 6, ypadding=5)
        table.attach(self.create_label(self.hip), 0, 1, 6, 7, ypadding=5)
        table.attach(self.create_label(self.ip), 1, 2, 6, 7, ypadding=5)
        table.attach(self.create_label(self.hmac), 0, 1, 7, 8, ypadding=5)
        table.attach(self.create_label(self.mac), 1, 2, 7, 8, ypadding=5)
        table.attach(self.create_label(self.hwarranty), 0, 1, 8, 9, ypadding=5)
        table.attach(self.create_label(self.warranty), 1, 2, 8, 9, ypadding=5)
        table.attach(self.create_label(self.hldrives), 0, 1, 9, 10, ypadding=5)
        table.attach(self.create_label(self.ldrives), 1, 2, 10, 11, ypadding=5)
        table.attach(self.create_label(self.hserial), 0, 1, 11, 12, ypadding=5)
        table.attach(self.create_label(self.serial), 1, 2, 12, 13, ypadding=5)

        table.set_homogeneous(False)

        # Show the table
        self.vbox.pack_start(table, True, True, 0)

        # Clipboard
        self.clipboard = gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)

        self.buttons = gtk.HBox(False, 0)

        # Copy button
        self.button1 = gtk.Button("Copy the info to clipboard")
        self.button1.connect("clicked", self.set_clipboard)
        self.button_fixed1 = gtk.Fixed()
        self.button_fixed1.put(self.button1, 70, 10)
        self.buttons.pack_start(self.button_fixed1, False, False, 0)

        # Check for updates button
        self.button2 = gtk.Button("Check for updates")
        self.button2.connect("clicked", self.check_updates)
        self.button_fixed2 = gtk.Fixed()
        self.button_fixed2.put(self.button2, 100, 10)
        self.buttons.pack_start(self.button_fixed2, False, False, 0)

        self.vbox.add(self.buttons)

        # Shows the window
        self.window.show_all()

    def create_label(self, content1):

        label = gtk.Label(content1)
        label.set_alignment(0, 0.5)
        return label

    def set_clipboard(self, widget):
        self.clipboard.set_text(self.hname + self.name + "\n" + self.hcpu + self.cpu + "\n" + self.hmem + self.mem + "\n" + self.hosv + self.osv + "\n" + self.hlupdate + self.lupdate + "\n" + self.hfqdn + self.fqdn + "\n" + self.hip + self.ip + "\n" + self.hmac + self.mac + "\n" + self.hldrives + "\n" + self.ldrives + "\n", -1)
        self.clipboard.store()
        return
    
    def check_updates(self, widget):
        subprocess.Popen('/usr/bin/update-manager %s', shell = True)
        return

def main():
    gtk.main()
    return 0

if __name__ == "__main__":
    InfoWin()
    main()

