#!/usr/bin/python3
# -*- coding: utf-8 -*-
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Program: HY install printers
# Written by Antti Luukkonen <antti.luukkonen@helsinki.fi>
# 23.3.2017
# JJ: ported to python 3

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk as gtk
from gi.repository import Gtk as Gtk
import subprocess
import re
import os

class InstallPrinter:

    def create_list(self):
        try:
            adprinters_raw = subprocess.check_output("ldapsearch -o ldif-wrap=no -o nettimeout=3 -b OU=rps,OU=canon,OU=tulostus,OU=mvPalvelut,DC=ad,DC=helsinki,DC=fi objectClass=printQueue location 2> /dev/null |grep -i -B 2 location", shell=True).decode("utf-8")
        except Exception:
            adprinters_raw = subprocess.check_output("cat /var/cache/cubbli-printers/printers.list", shell=True).decode("utf-8")
        adprinters_fixed = ""
        for result in adprinters_raw.split("--"):
            for line in result.split("\n"):
                if line.startswith("location::"):
                    location_match = re.search('location:: ([\w|\d|=]+)', line)
                    command = "echo -n " + location_match.group(1) + " | base64 -d"
                    line = "location: " + subprocess.check_output(command, shell=True).decode("utf-8")
                adprinters_fixed += "\n" + line
        return adprinters_fixed

# AL: This gives error message with some search words, but still seems to work

    def search_ad(self, widget):
        # empty the treeview
        self.listmodel.clear()
        # get text from search field and change text to lowercase
        text = self.entry.get_text()
        entry_text = text.lower().strip()
        # check if user is searching with serial number or address
        if entry_text.startswith("mx") or entry_text.startswith("mm") or entry_text.startswith("mk") or entry_text.startswith("ly") or entry_text.startswith("nz"):
            search_results = re.findall("(CN=.*" + entry_text + ".*)\r?\n(.*)", self.adprinters, re.M + re.IGNORECASE)
            for row in search_results:
                printer_info = self.get_data(row)
                self.listmodel.append(printer_info)
        else:
            search_results = re.findall("(.*)\r?\n(^location:.*" + entry_text + ".*)", self.adprinters, re.M + re.IGNORECASE)
            for row in search_results:
                printer_info = self.get_data(row)
                self.listmodel.append(printer_info)
        return

    def get_data(self, row):
        name = re.search("\d-([\w]+),", row[0])
        location = re.search("location: (.*[\w|\d]+)", row[1])
        return [name.group(1), location.group(1)]

    def add_printer(self, adprinters_fixed):
        # get selection
        selection = self.view.get_selection()
        (model, pathlist) = selection.get_selected_rows()
        # get values from the cells and use lpadmin to add printer to the system
        if pathlist:
            printer = model.get_value(model.get_iter(pathlist[0][0]), 0)
            printer_server = re.search("(\w.*)-" + printer, self.adprinters)
            os.system('/usr/sbin/lpadmin -p ' + printer + ' -v krbsmb://ATKK/' + printer_server.group(1) + '.ad.helsinki.fi/' + printer + ' -P /usr/share/ppd/cubbli/cel-iradvc5250-ps-en-jj.ppd -E')
            # Pop up a window to ask about default printer setting
            popup = gtk.MessageDialog(None, gtk.DialogFlags.MODAL, Gtk.MessageType.QUESTION, gtk.ButtonsType.YES_NO, None)
            popup.set_markup("Printer " + printer + " installed successfully.\nSet as default printer?")
            if popup.run() == gtk.ResponseType.YES:
                os.system('/usr/sbin/lpadmin -d ' + printer)
            popup.destroy()
        return

    def __init__(self):

        # get all the printers from AD to a variable
        self.adprinters = self.create_list()

        # create the window
        app_window = gtk.Window()
        app_window.set_title("HY Install Printers")

        # close the window
        app_window.connect("destroy", lambda w: gtk.main_quit())

        # size of the window
        app_window.set_size_request(700, 400)
        #app_window.set_position(gtk.WIN_POS_CENTER)

        # create VBOX for the search field
        vbox = gtk.VBox(False, 0)

        # create HBOX for the buttons
        buttons = gtk.HBox(False, 0)

        # create the search field with max lenght and preset text
        self.entry = gtk.Entry()
        self.entry.set_max_length(72)
        self.entry.connect("activate", self.search_ad)
        self.entry.set_text("Printer serial number or location")
        vbox.pack_start(self.entry, False, False, 0)

        # create the "search" button
        button1 = gtk.Button("Search")
        button1.connect("released", self.search_ad)
        button1_fixed = gtk.Fixed()
        button1_fixed.put(button1, 5, 5)
        buttons.pack_start(button1_fixed, False, False, 0)

        # create the "close" button
        button2 = gtk.Button("Close")
        button2.connect("released", lambda w: gtk.main_quit())
        button2_fixed = gtk.Fixed()
        button2_fixed.put(button2, 5, 5)
        buttons.pack_start(button2_fixed, False, False, 0)

        # create the "add" button
        button3 = gtk.Button("Add printer")
        button3.connect("released", self.add_printer)
        button3_fixed = gtk.Fixed()
        button3_fixed.put(button3, 5, 5)
        buttons.pack_start(button3_fixed, False, False, 0)

        # add buttons to the VBOX
        vbox.add(buttons)

        # create the liststore with 2 columns for strings
        self.listmodel = gtk.ListStore(str, str)

        # create the treeview using liststore
        self.view = gtk.TreeView(self.listmodel)
        self.view.set_size_request(100, 320)

        # create the columns with text
        col1 = gtk.TreeViewColumn('Serial number')
        col2 = gtk.TreeViewColumn('Location')

        # add columns to treeview
        self.view.append_column(col1)
        self.view.append_column(col2)

        # create a CellRenderers to render the data
        cell1 = gtk.CellRendererText()
        cell2 = gtk.CellRendererText()

        # adds the cells to the columns
        col1.pack_start(cell1, True)
        col2.pack_start(cell2, True)

        # text=number chooses the text field from listmodel.append
        col1.set_attributes(cell1, text=0)
        col2.set_attributes(cell2, text=1)

        # make treeview searchable
        self.view.set_search_column(0)

        # enable column sorting
        col1.set_sort_column_id(0)
        col2.set_sort_column_id(1)

        # make treeview scrollable
        scrolled = gtk.ScrolledWindow()
        scrolled.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        scrolled.add(self.view)

        # add treeview to VBOX
        vbox.add(scrolled)

        # add VBOX to the WINDOW
        app_window.add(vbox)

        # show all
        app_window.show_all()
        return

def main():
    gtk.main()
    return 0

if __name__ == "__main__":
    InstallPrinter()
    main()
