#!/usr/bin/env python
# -*- coding: utf-8 -*-
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Program: HY install software
# Written by Antti Luukkonen <antti.luukkonen@helsinki.fi>
# 23.3.2017

import gtk
import os
import subprocess

class InstallSoftware:

    # path of whitelist
    # rows contain package name and description:
    # matlab Matlab 2016a (15 GB free space recommended)
    swlist_path = '/etc/cubbli/cubbli-apt-wrapper/allowed_packages'
    
    # free space on root filesystem
    freedisk = subprocess.check_output("df -h / | awk 'FNR == 2 {print $4}'", shell=True)

    def update_list(self, collection):
        try:
            swlist = open(self.swlist_path).readlines()
        except IOError:
            swlist = []

        for sw in swlist:
            sw_package, description = sw.strip().partition(' ')[::2]

            if not description:
                description = sw_package

            # Append the item to the list
            collection.append([description, sw_package])

    def install_sw(self, widget):
        selection = self.view.get_selection()
        (selected, number) = selection.get_selected_rows()
        sw_name = selected.get_value(selected.get_iter(number[0][0]), 0)
        package = selected.get_value(selected.get_iter(number[0][0]), 1)
        pkgsize_raw = subprocess.check_output("apt-cache show " + package + "| grep -i ^size | awk 'FNR == 1 {print $2}'", shell=True)
        pkgsize = int(pkgsize_raw) / 1000000
        if pkgsize == 0:
            pkgsize = "less than 1"
        popup = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_QUESTION, gtk.BUTTONS_OK_CANCEL, None)
        popup.set_markup("Do you want to install the following software:\n\n" + sw_name + "\n\nDisk space required: " + str(pkgsize) + "MB\nFree disk space: " + self.freedisk)
        if popup.run() == gtk.RESPONSE_OK:
            os.system('/usr/sbin/cubbli-apt-wrapper ' + 'install ' + (package.lower()))
        popup.destroy()
        return 0

    def remove_sw(self, widget):
        selection = self.view.get_selection()
        (selected, number) = selection.get_selected_rows()
        sw_name = selected.get_value(selected.get_iter(number[0][0]), 0)
        package = selected.get_value(selected.get_iter(number[0][0]), 1)
        popup = gtk.MessageDialog(None, gtk.DIALOG_MODAL, gtk.MESSAGE_QUESTION, gtk.BUTTONS_OK_CANCEL, None)
        popup.set_markup("Do you want to remove the following software:\n\n" + sw_name)
        if popup.run() == gtk.RESPONSE_OK:
            os.system('/usr/sbin/cubbli-apt-wrapper ' + 'remove ' + (package.lower()))
        popup.destroy()
        return 0
    
    def __init__(self):

        # create the liststore with 2 strings
        collection = gtk.ListStore(str, str)

        # update the software collection to the list
        self.update_list(collection)

        # create the treeview using collection
        self.view = gtk.TreeView(collection)
        self.view.set_size_request(100, 240)

        # create the window
        app_window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        app_window.set_title("HY Install Software")
        
        # close the window
        app_window.connect("destroy", lambda w: gtk.main_quit())

        # size of the window
        app_window.set_size_request(400, 300)
        app_window.set_position(gtk.WIN_POS_CENTER)

        # create VBOX for the software field
        vbox = gtk.VBox(False, 0)
            
        # create HBOX for the buttons
        buttons = gtk.HBox(False, 0)

        # create the "search" button
        button1 = gtk.Button("Install")
        button1.connect("released", self.install_sw)
        button1_fixed = gtk.Fixed()
        button1_fixed.put(button1, 0, 10)
        buttons.pack_start(button1_fixed, False, False, 10)

        # create the "close" button
        button2 = gtk.Button("Close")
        button2.connect("released", lambda w: gtk.main_quit())
        button2_fixed = gtk.Fixed()
        button2_fixed.put(button2, 0, 10)
        buttons.pack_start(button2_fixed, False, False, 105)

        # create the "remove" button
        button3 = gtk.Button("Remove")
        button3.connect("released", self.remove_sw)
        button3_fixed = gtk.Fixed()
        button3_fixed.put(button3, 0, 10)
        buttons.pack_end(button3_fixed, False, False, 10)

        # add buttons to the VBOX
        vbox.add(buttons)

        # create the columns with text
        col1 = gtk.TreeViewColumn('Software')
        col1.set_alignment(0.5)
        #col2 = gtk.TreeViewColumn('Version')

        # add columns to treeview
        self.view.append_column(col1)
        #self.view.append_column(col2)

        # create a CellRenderers to render the data
        cell1 = gtk.CellRendererText()
        cell1.set_alignment(0, 1)
        #cell2 = gtk.CellRendererText()

        # adds the cells to the columns
        col1.pack_start(cell1, True)
        #col2.pack_start(cell2, True)

        # text=number chooses the text field from collection.append
        col1.set_attributes(cell1, text=0)
        #col2.set_attributes(cell2, text=1)
        
        # make treeview searchable
        self.view.set_search_column(0)

        # enable column sorting
        col1.set_sort_column_id(0)
        #col2.set_sort_column_id(1)

        # make treeview scrollable
        scrolled = gtk.ScrolledWindow()
        scrolled.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        scrolled.add(self.view)

        # add treeview to VBOX
        vbox.add(scrolled)

        # add VBOX to the WINDOW
        app_window.add(vbox)

        # show all
        app_window.show_all()
        return

def main():
    gtk.main()
    return 0

if __name__ == "__main__":
    InstallSoftware()
    main()